@@ -1,7 +1,7 @@
 /*  zint.h - definitions for libzint */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -31,8 +31,10 @@
 /* SPDX-License-Identifier: BSD-3-Clause */
 
 /*
- * For version, see "zintconfig.h"
- * For documentation, see "../docs/manual.txt"
+ * Version: 2.15.0 (see "zintconfig.h")
+ *
+ * For documentation, see "../docs/manual.txt" or "../docs/manual.html" or online at
+ * https://zint.org.uk/manual/
  */
 
 #ifndef ZINT_H
@@ -42,34 +44,37 @@
 extern "C" {
 #endif /* __cplusplus */
 
+    /* Vector elements - see vector header `zint_vector` below */
     struct zint_vector_rect {
-        float x, y, height, width;
+        float x, y;         /* Top left */
+        float height, width;
         int colour;         /* -1 for foreground, 1-8 for Cyan, Blue, Magenta, Red, Yellow, Green, Black, White */
         struct zint_vector_rect *next; /* Pointer to next rectangle */
     };
 
     struct zint_vector_hexagon {
-        float x, y, diameter;
-        int rotation;       /* 0, 90, 180, 270 degrees */
+        float x, y;         /* Centre */
+        float diameter;     /* Short (minimal) diameter (i.e. diameter of inscribed circle) */
+        int rotation;       /* 0, 90, 180, 270 degrees, where 0 has apex at top, i.e. short diameter is horizontal */
         struct zint_vector_hexagon *next; /* Pointer to next hexagon */
     };
 
     struct zint_vector_string {
-        float x, y;         /* x, y position relative to halign */
+        float x, y;         /* x is relative to halign (i.e. centre, left, right), y is relative to baseline */
         float fsize;        /* Font size */
-        float width;        /* Suggested string width, may be 0 if none recommended */
-        int length;         /* Number of characters */
+        float width;        /* Rendered width estimate */
+        int length;         /* Number of characters (bytes) */
         int rotation;       /* 0, 90, 180, 270 degrees */
         int halign;         /* Horizontal alignment: 0 for centre, 1 for left, 2 for right (end) */
-        unsigned char *text;
+        unsigned char *text; /* UTF-8, NUL-terminated */
         struct zint_vector_string *next; /* Pointer to next string */
     };
 
     struct zint_vector_circle {
-        float x, y;
+        float x, y;         /* Centre */
         float diameter;     /* Circle diameter. Does not include width (if any) */
         float width;        /* Width of circle perimeter (circumference). 0 for fill (disc) */
-        int colour;         /* Non-zero for draw with background colour (else draw with foreground colour) */
+        int colour;         /* Zero for draw with foreground colour (else draw with background colour (legacy)) */
         struct zint_vector_circle *next; /* Pointer to next circle */
     };
 
@@ -82,7 +87,7 @@ extern "C" {
         struct zint_vector_circle *circles; /* Pointer to first circle */
     };
 
-    /* Structured Append info - ignored unless `zint_structapp.count` is set to non-zero value */
+    /* Structured Append info (see `symbol->structapp` below) - ignored unless `zint_structapp.count` is non-zero */
     struct zint_structapp {
         int index;          /* Position in Structured Append sequence, 1-based. Must be <= `count` */
         int count;          /* Number of symbols in Structured Append sequence. Set >= 2 to add SA Info */
@@ -98,8 +103,8 @@ extern "C" {
         int whitespace_height; /* Height in X-dimensions of whitespace above & below the barcode */
         int border_width;   /* Size of border in X-dimensions */
         int output_options; /* Various output parameters (bind, box etc, see below) */
-        char fgcolour[10];  /* Foreground as RGB/RGBA hexadecimal string, 6 or 8 characters, NUL-terminated */
-        char bgcolour[10];  /* Background as RGB/RGBA hexadecimal string, 6 or 8 characters, NUL-terminated */
+        char fgcolour[16];  /* Foreground as hexadecimal RGB/RGBA or decimal "C,M,Y,K" string, NUL-terminated */
+        char bgcolour[16];  /* Background as hexadecimal RGB/RGBA or decimal "C,M,Y,K" string, NUL-terminated */
         char *fgcolor;      /* Pointer to fgcolour (alternate spelling) */
         char *bgcolor;      /* Pointer to bgcolour (alternate spelling) */
         char outfile[256];  /* Name of file to output to, NUL-terminated. Default "out.png" ("out.gif" if no PNG) */
@@ -108,16 +113,17 @@ extern "C" {
         int option_2;       /* Symbol-specific options */
         int option_3;       /* Symbol-specific options */
         int show_hrt;       /* Show (1) or hide (0) Human Readable Text (HRT). Default 1 */
-        int fontsize;       /* Unused */
         int input_mode;     /* Encoding of input data (see DATA_MODE etc below). Default DATA_MODE */
         int eci;            /* Extended Channel Interpretation. Default 0 (none) */
         float dpmm;         /* Resolution of output in dots per mm (BMP/EMF/PCX/PNG/TIF only). Default 0 (none) */
         float dot_size;     /* Size of dots used in BARCODE_DOTTY_MODE. Default 0.8 */
+        float text_gap;     /* Gap between barcode and text (HRT) in X-dimensions. Default 1 */
         float guard_descent; /* Height in X-dimensions that EAN/UPC guard bars descend. Default 5 */
         struct zint_structapp structapp; /* Structured Append info. Default structapp.count 0 (none) */
         int warn_level;     /* Affects error/warning value returned by Zint API (see WARN_XXX below) */
         int debug;          /* Debugging flags */
-        unsigned char text[128]; /* Human Readable Text (HRT) (if any), UTF-8, NUL-terminated (output only) */
+        unsigned char text[256]; /* Human Readable Text (HRT) (if any), UTF-8, NUL-terminated (output only) */
+        int text_length;    /* Length of text in bytes (output only) */
         int rows;           /* Number of rows used by the symbol (output only) */
         int width;          /* Width of the generated symbol (output only) */
         unsigned char encoded_data[200][144]; /* Encoded data (output only). Allows for rows of 1152 modules */
@@ -127,8 +133,9 @@ extern "C" {
         int bitmap_width;   /* Width of bitmap image (raster output only) */
         int bitmap_height;  /* Height of bitmap image (raster output only) */
         unsigned char *alphamap; /* Array of alpha values used (raster output only) */
-        unsigned int bitmap_byte_length; /* Size of BMP bitmap data (raster output only) */
         struct zint_vector *vector; /* Pointer to vector header (vector output only) */
+        unsigned char *memfile; /* Pointer to in-memory file buffer if BARCODE_MEMORY_FILE (output only) */
+        int memfile_size;   /* Length of in-memory file buffer (output only) */
     };
 
     /* Segment for use with `ZBarcode_Encode_Segs()` below */
@@ -185,7 +192,7 @@ extern "C" {
 #define BARCODE_PDF417TRUNC     56  /* Legacy */
 #define BARCODE_MAXICODE        57  /* MaxiCode */
 #define BARCODE_QRCODE          58  /* QR Code */
-#define BARCODE_CODE128AB       60  /* Code 128 (Suppress subset C) */
+#define BARCODE_CODE128AB       60  /* Code 128 (Suppress Code Set C) */
 #define BARCODE_CODE128B        60  /* Legacy */
 #define BARCODE_AUSPOST         63  /* Australia Post Standard Customer */
 #define BARCODE_AUSREPLY        66  /* Australia Post Reply Paid */
@@ -268,26 +275,30 @@ extern "C" {
 #define BARCODE_ULTRA           144 /* Ultracode */
 #define BARCODE_RMQR            145 /* Rectangular Micro QR Code (rMQR) */
 #define BARCODE_BC412           146 /* IBM BC412 (SEMI T1-95) */
-#define BARCODE_LAST            146 /* Max barcode number marker, not barcode */
+#define BARCODE_DXFILMEDGE      147 /* DX Film Edge Barcode on 35mm and APS films */
+#define BARCODE_LAST            147 /* Max barcode number marker, not barcode */
 
 /* Output options (`symbol->output_options`) */
-#define BARCODE_BIND_TOP        0x0001  /* Boundary bar above the symbol only (not below), does not affect stacking */
+#define BARCODE_BIND_TOP        0x00001 /* Boundary bar above the symbol only (not below), does not affect stacking */
                                         /* Note: value was once used by the legacy (never-used) BARCODE_NO_ASCII */
-#define BARCODE_BIND            0x0002  /* Boundary bars above & below the symbol and between stacked symbols */
-#define BARCODE_BOX             0x0004  /* Box around symbol */
-#define BARCODE_STDOUT          0x0008  /* Output to stdout */
-#define READER_INIT             0x0010  /* Reader Initialisation (Programming) */
-#define SMALL_TEXT              0x0020  /* Use smaller font */
-#define BOLD_TEXT               0x0040  /* Use bold font */
-#define CMYK_COLOUR             0x0080  /* CMYK colour space (Encapsulated PostScript and TIF) */
-#define BARCODE_DOTTY_MODE      0x0100  /* Plot a matrix symbol using dots rather than squares */
-#define GS1_GS_SEPARATOR        0x0200  /* Use GS instead of FNC1 as GS1 separator (Data Matrix) */
-#define OUT_BUFFER_INTERMEDIATE 0x0400  /* Return ASCII values in bitmap buffer (OUT_BUFFER only) */
-#define BARCODE_QUIET_ZONES     0x0800  /* Add compliant quiet zones (additional to any specified whitespace) */
+#define BARCODE_BIND            0x00002 /* Boundary bars above & below the symbol and between stacked symbols */
+#define BARCODE_BOX             0x00004 /* Box around symbol */
+#define BARCODE_STDOUT          0x00008 /* Output to stdout */
+#define READER_INIT             0x00010 /* Reader Initialisation (Programming) */
+#define SMALL_TEXT              0x00020 /* Use smaller font */
+#define BOLD_TEXT               0x00040 /* Use bold font */
+#define CMYK_COLOUR             0x00080 /* CMYK colour space (Encapsulated PostScript and TIF) */
+#define BARCODE_DOTTY_MODE      0x00100 /* Plot a matrix symbol using dots rather than squares */
+#define GS1_GS_SEPARATOR        0x00200 /* Use GS instead of FNC1 as GS1 separator (Data Matrix) */
+#define OUT_BUFFER_INTERMEDIATE 0x00400 /* Return ASCII values in bitmap buffer (OUT_BUFFER only) */
+#define BARCODE_QUIET_ZONES     0x00800 /* Add compliant quiet zones (additional to any specified whitespace) */
                                         /* Note: CODE16K, CODE49, CODABLOCKF, ITF14, EAN/UPC have default quiet zones
                                          */
-#define BARCODE_NO_QUIET_ZONES  0x1000  /* Disable quiet zones, notably those with defaults as listed above */
-#define COMPLIANT_HEIGHT        0x2000  /* Warn if height not compliant and use standard height (if any) as default */
+#define BARCODE_NO_QUIET_ZONES  0x01000 /* Disable quiet zones, notably those with defaults as listed above */
+#define COMPLIANT_HEIGHT        0x02000 /* Warn if height not compliant, or use standard height (if any) as default */
+#define EANUPC_GUARD_WHITESPACE 0x04000 /* Add quiet zone indicators ("<"/">") to HRT whitespace (EAN/UPC) */
+#define EMBED_VECTOR_FONT       0x08000 /* Embed font in vector output - currently only for SVG output */
+#define BARCODE_MEMORY_FILE     0x10000 /* Write output to in-memory buffer `memfile` instead of to `outfile` */
 
 /* Input data types (`symbol->input_mode`) */
 #define DATA_MODE               0       /* Binary */
@@ -300,10 +311,13 @@ extern "C" {
 #define HEIGHTPERROW_MODE       0x0040  /* Interpret `height` as per-row rather than as overall height */
 #define FAST_MODE               0x0080  /* Use faster if less optimal encodation or other shortcuts if available */
                                         /* Note: affects DATAMATRIX, MICROPDF417, PDF417, QRCODE & UPNQR only */
+#define EXTRA_ESCAPE_MODE       0x0100  /* Process special symbology-specific escape sequences as well as others */
+                                        /* Note: currently Code 128 only */
 
 /* Data Matrix specific options (`symbol->option_3`) */
 #define DM_SQUARE               100     /* Only consider square versions on automatic symbol size selection */
 #define DM_DMRE                 101     /* Consider DMRE versions on automatic symbol size selection */
+#define DM_ISO_144              128     /* Use ISO instead of "de facto" format for 144x144 (i.e. don't skew ECC) */
 
 /* QR, Han Xin, Grid Matrix specific options (`symbol->option_3`) */
 #define ZINT_FULL_MULTIBYTE     200     /* Enable Kanji/Hanzi compression for Latin-1 & binary data */
@@ -312,6 +326,7 @@ extern "C" {
 #define ULTRA_COMPRESSION       128     /* Enable Ultracode compression (experimental) */
 
 /* Warning and error conditions (API return values) */
+#define ZINT_WARN_HRT_TRUNCATED     1   /* Human Readable Text was truncated (max 199 bytes) */
 #define ZINT_WARN_INVALID_OPTION    2   /* Invalid option given but overridden by Zint */
 #define ZINT_WARN_USES_ECI          3   /* Automatic ECI inserted by Zint */
 #define ZINT_WARN_NONCOMPLIANT      4   /* Symbol created not compliant with standards */
@@ -326,6 +341,7 @@ extern "C" {
 #define ZINT_ERROR_FILE_WRITE       12  /* Error writing to output file */
 #define ZINT_ERROR_USES_ECI         13  /* Error counterpart of warning if WARN_FAIL_ALL set (see below) */
 #define ZINT_ERROR_NONCOMPLIANT     14  /* Error counterpart of warning if WARN_FAIL_ALL set */
+#define ZINT_ERROR_HRT_TRUNCATED    15  /* Error counterpart of warning if WARN_FAIL_ALL set */
 
 /* Warning level (`symbol->warn_level`) */
 #define WARN_DEFAULT            0  /* Default behaviour */
@@ -334,7 +350,8 @@ extern "C" {
 /* Capability flags (ZBarcode_Cap() `cap_flag`) */
 #define ZINT_CAP_HRT                0x0001  /* Prints Human Readable Text? */
 #define ZINT_CAP_STACKABLE          0x0002  /* Is stackable? */
-#define ZINT_CAP_EXTENDABLE         0x0004  /* Is extendable with add-on data? (Is EAN/UPC?) */
+#define ZINT_CAP_EANUPC             0x0004  /* Is EAN/UPC? */
+#define ZINT_CAP_EXTENDABLE         0x0004  /* Legacy */
 #define ZINT_CAP_COMPOSITE          0x0008  /* Can have composite data? */
 #define ZINT_CAP_ECI                0x0010  /* Supports Extended Channel Interpretations? */
 #define ZINT_CAP_GS1                0x0020  /* Supports GS1 data? */
@@ -364,6 +381,8 @@ extern "C" {
 #  else
 #    define ZINT_EXTERN extern
 #  endif
+#elif defined(__GNUC__) && __GNUC__ >= 4
+#  define ZINT_EXTERN extern __attribute__((__visibility__("default")))
 #else
 #  define ZINT_EXTERN extern
 #endif
@@ -374,6 +393,9 @@ extern "C" {
     /* Free any output buffers that may have been created and initialize output fields */
     ZINT_EXTERN void ZBarcode_Clear(struct zint_symbol *symbol);
 
+    /* Free any output buffers that may have been created and reset all fields to defaults */
+    ZINT_EXTERN void ZBarcode_Reset(struct zint_symbol *symbol);
+
     /* Free a symbol structure, including any output buffers */
     ZINT_EXTERN void ZBarcode_Delete(struct zint_symbol *symbol);
 
@@ -441,7 +463,7 @@ extern "C" {
     ZINT_EXTERN int ZBarcode_ValidID(int symbol_id);
 
     /* Copy BARCODE_XXX name of `symbol_id` into `name` buffer, NUL-terminated.
-       Returns 0 if valid, non-zero (1 or -1) if not valid */
+       Returns 0 if valid, 1 if not valid */
     ZINT_EXTERN int ZBarcode_BarcodeName(int symbol_id, char name[32]);
 
     /* Return the capability flags for symbology `symbol_id` that match `cap_flag` */
